@@ -1,4 +1,4 @@
-/*	$NetBSD: fortune.c,v 1.8 1995/03/23 08:28:40 cgd Exp $	*/
+/*	$NetBSD: fortune.c,v 1.23 2000/04/14 05:58:02 simonb Exp $	*/
 
 /*-
  * Copyright (c) 1986, 1993
@@ -36,22 +36,23 @@
  * SUCH DAMAGE.
  */
 
+#include <sys/cdefs.h>
 #ifndef lint
-static char copyright[] =
-"@(#) Copyright (c) 1986, 1993\n\
-	The Regents of the University of California.  All rights reserved.\n";
+__COPYRIGHT("@(#) Copyright (c) 1986, 1993\n\
+	The Regents of the University of California.  All rights reserved.\n");
 #endif /* not lint */
 
 #ifndef lint
 #if 0
 static char sccsid[] = "@(#)fortune.c	8.1 (Berkeley) 5/31/93";
 #else
-static char rcsid[] = "$NetBSD: fortune.c,v 1.8 1995/03/23 08:28:40 cgd Exp $";
+__RCSID("$NetBSD: fortune.c,v 1.23 2000/04/14 05:58:02 simonb Exp $");
 #endif
 #endif /* not lint */
 
 # include	<sys/param.h>
 # include	<sys/stat.h>
+# include	<sys/endian.h>
 
 # include	<dirent.h>
 # include	<fcntl.h>
@@ -61,6 +62,8 @@ static char rcsid[] = "$NetBSD: fortune.c,v 1.8 1995/03/23 08:28:40 cgd Exp $";
 # include	<ctype.h>
 # include	<stdlib.h>
 # include	<string.h>
+# include	<err.h>
+# include	<time.h>
 # include	"strfile.h"
 # include	"pathnames.h"
 
@@ -83,17 +86,13 @@ static char rcsid[] = "$NetBSD: fortune.c,v 1.8 1995/03/23 08:28:40 cgd Exp $";
 # define	NDEBUG	1
 # endif
 
-#ifdef linux
-#define d_namlen d_reclen
-#endif
-
 typedef struct fd {
 	int		percent;
 	int		fd, datfd;
 	off_t		pos;
 	FILE		*inf;
-	char		*name;
-	char		*path;
+	const char	*name;
+	const char	*path;
 	char		*datfile, *posfile;
 	bool		read_tbl;
 	bool		was_pos_file;
@@ -122,7 +121,7 @@ char	*Fortbuf = NULL;			/* fortune buffer for -m */
 
 int	Fort_len = 0;
 
-off_t	Seekpts[2];			/* seek pointers to fortunes */
+u_int64_t	Seekpts[2];			/* seek pointers to fortunes */
 
 FILEDESC	*File_list = NULL,	/* Head of file list */
 		*File_tail = NULL;	/* Tail of file list */
@@ -132,9 +131,9 @@ STRFILE		Noprob_tbl;		/* sum of data for all no prob files */
 
 int	 add_dir __P((FILEDESC *));
 int	 add_file __P((int,
-	    char *, char *, FILEDESC **, FILEDESC **, FILEDESC *));
-void	 all_forts __P((FILEDESC *, char *));
-char	*copy __P((char *, u_int));
+	    const char *, const char *, FILEDESC **, FILEDESC **, FILEDESC *));
+void	 all_forts __P((FILEDESC *, const char *));
+char	*copy __P((const char *, u_int));
 void	 display __P((FILEDESC *));
 void	 do_free __P((void *));
 void	*do_malloc __P((u_int));
@@ -145,13 +144,14 @@ void	 get_pos __P((FILEDESC *));
 void	 get_tbl __P((FILEDESC *));
 void	 getargs __P((int, char *[]));
 void	 init_prob __P((void));
-int	 is_dir __P((char *));
-int	 is_fortfile __P((char *, char **, char **, int));
-int	 is_off_name __P((char *));
+int	 is_dir __P((const char *));
+int	 is_fortfile __P((const char *, char **, char **, int));
+int	 is_off_name __P((const char *));
+int	 main __P((int, char *[]));
 int	 max __P((int, int));
 FILEDESC *
 	 new_fp __P((void));
-char	*off_name __P((char *));
+char	*off_name __P((const char *));
 void	 open_dat __P((FILEDESC *));
 void	 open_fp __P((FILEDESC *));
 FILEDESC *
@@ -160,7 +160,7 @@ void	 print_file_list __P((void));
 void	 print_list __P((FILEDESC *, int));
 void	 sum_noprobs __P((FILEDESC *));
 void	 sum_tbl __P((STRFILE *, STRFILE *));
-void	 usage __P((void));
+void	 usage __P((void)) __attribute__((__noreturn__));
 void	 zero_tbl __P((STRFILE *));
 
 #ifndef	NO_REGEX
@@ -171,31 +171,56 @@ int	 maxlen_in_list __P((FILEDESC *));
 #endif
 
 #ifndef NO_REGEX
-#ifdef REGCMP
-# define	RE_COMP(p)	(Re_pat = regcmp(p, NULL))
-# define	BAD_COMP(f)	((f) == NULL)
-# define	RE_EXEC(p)	regex(Re_pat, (p))
+# if HAVE_REGCMP
+#  define	RE_INIT()
+#  define	RE_COMP(p)	(Re_pat = regcmp(p, NULL))
+#  define	BAD_COMP(f)	((f) == NULL)
+#  define	RE_EXEC(p)	regex(Re_pat, (p))
+#  define	RE_FREE()
 
 char	*Re_pat;
 
 char	*regcmp(), *regex();
-#else
-# define	RE_COMP(p)	(p = re_comp(p))
-# define	BAD_COMP(f)	((f) != NULL)
-# define	RE_EXEC(p)	re_exec(p)
-
+# elif HAVE_RE_COMP
+#  define	RE_INIT()
+#  define	RE_COMP(p)	(p = re_comp(p))
+#  define	BAD_COMP(f)	((f) != NULL)
+#  define	RE_EXEC(p)	re_exec(p)
+#  define	RE_FREE()
+# elif HAVE_REGCOMP
+#  include <regex.h>
+regex_t *Re_pat = NULL;
+#  define	RE_INIT()	if (Re_pat == NULL && \
+				    (Re_pat = calloc(sizeof(*Re_pat), 1)) == NULL)\
+					err(1, NULL)
+#  define	RE_COMP(p)	(regcomp(Re_pat, p, REG_EXTENDED))
+#  define	BAD_COMP(f)	((f) != 0)
+#  define	RE_EXEC(p)	(!regexec(Re_pat, p, 0, NULL, 0))
+#  define	RE_FREE()	if (Re_pat != NULL) \
+					regfree(Re_pat), Re_pat = NULL
+# else
+	#error "Need to define HAVE_REGCMP, HAVE_RE_COMP, or HAVE_REGCOMP"
+# endif
 #endif
+
+#if (defined(__linux__) && !defined(__GLIBC__)) || (defined(__GLIBC__) && !defined(_DIRENT_HAVE_D_NAMLEN))
+#define		NAMLEN(d)	(strlen((d)->d_name))
+#else
+#define		NAMLEN(d)	((d)->d_namlen)
 #endif
 
 int
 main(ac, av)
-int	ac;
-char	*av[];
+	int	ac;
+	char	*av[];
 {
 #ifdef	OK_TO_WRITE_DISK
 	int	fd;
 #endif	/* OK_TO_WRITE_DISK */
 
+	/* Revoke setgid privileges */
+	setregid(getgid(), getgid());
+
 	getargs(ac, av);
 
 #ifndef NO_REGEX
@@ -213,10 +238,8 @@ char	*av[];
 	display(Fortfile);
 
 #ifdef	OK_TO_WRITE_DISK
-	if ((fd = creat(Fortfile->posfile, 0666)) < 0) {
-		perror(Fortfile->posfile);
-		exit(1);
-	}
+	if ((fd = creat(Fortfile->posfile, 0666)) < 0)
+		err(1, "Can't create `%s'", Fortfile->posfile);
 #ifdef	LOCK_EX
 	/*
 	 * if we can, we exclusive lock, but since it isn't very
@@ -237,27 +260,27 @@ char	*av[];
 			(void) fortlen();
 		sleep((unsigned int) max(Fort_len / CPERS, MINW));
 	}
-	exit(0);
-	/* NOTREACHED */
+	return(0);
 }
 
 void
 display(fp)
-FILEDESC	*fp;
+	FILEDESC	*fp;
 {
-	register char	*p, ch;
+	char	*p, ch;
 	char	line[BUFSIZ];
 
 	open_fp(fp);
-	(void) fseek(fp->inf, (long)Seekpts[0], 0);
+	(void) fseek(fp->inf, (long)Seekpts[0], SEEK_SET);
 	for (Fort_len = 0; fgets(line, sizeof line, fp->inf) != NULL &&
 	    !STR_ENDSTRING(line, fp->tbl); Fort_len++) {
-		if (fp->tbl.str_flags & STR_ROTATED)
-			for (p = line; ch = *p; ++p)
+		if (fp->tbl.str_flags & STR_ROTATED) {
+			for (p = line; (ch = *p) != 0; ++p)
 				if (isupper(ch))
 					*p = 'A' + (ch - 'A' + 13) % 26;
 				else if (islower(ch))
 					*p = 'a' + (ch - 'a' + 13) % 26;
+		}
 		fputs(line, stdout);
 	}
 	(void) fflush(stdout);
@@ -270,14 +293,14 @@ FILEDESC	*fp;
 int
 fortlen()
 {
-	register int	nchar;
-	char		line[BUFSIZ];
+	int	nchar;
+	char	line[BUFSIZ];
 
 	if (!(Fortfile->tbl.str_flags & (STR_RANDOM | STR_ORDERED)))
 		nchar = (Seekpts[1] - Seekpts[0] <= SLEN);
 	else {
 		open_fp(Fortfile);
-		(void) fseek(Fortfile->inf, (long)Seekpts[0], 0);
+		(void) fseek(Fortfile->inf, (long)Seekpts[0], SEEK_SET);
 		nchar = 0;
 		while (fgets(line, sizeof line, Fortfile->inf) != NULL &&
 		       !STR_ENDSTRING(line, Fortfile->tbl))
@@ -292,23 +315,21 @@ fortlen()
  */
 void
 getargs(argc, argv)
-register int	argc;
-register char	**argv;
+	int	argc;
+	char	**argv;
 {
-	register int	ignore_case;
+	int	ignore_case;
 # ifndef NO_REGEX
-	register char	*pat = NULL;
+	char	*pat = NULL;
 # endif	/* NO_REGEX */
-	extern char *optarg;
-	extern int optind;
 	int ch;
 
 	ignore_case = FALSE;
 
 # ifdef DEBUG
-	while ((ch = getopt(argc, argv, "aDefilm:osw")) != EOF)
+	while ((ch = getopt(argc, argv, "aDefilm:osw")) != -1)
 #else
-	while ((ch = getopt(argc, argv, "aefilm:osw")) != EOF)
+	while ((ch = getopt(argc, argv, "aefilm:osw")) != -1)
 #endif /* DEBUG */
 		switch(ch) {
 		case 'a':		/* any fortune */
@@ -342,9 +363,7 @@ register char	**argv;
 # ifdef	NO_REGEX
 		case 'i':			/* case-insensitive match */
 		case 'm':			/* dump out the fortunes */
-			(void) fprintf(stderr,
-			    "fortune: can't match fortunes on this system (Sorry)\n");
-			exit(0);
+			errx(1, "Can't match fortunes on this system (Sorry)");
 # else	/* NO_REGEX */
 		case 'm':			/* dump out the fortunes */
 			Match++;
@@ -376,12 +395,14 @@ register char	**argv;
 	if (pat != NULL) {
 		if (ignore_case)
 			pat = conv_pat(pat);
+		RE_INIT();
 		if (BAD_COMP(RE_COMP(pat))) {
-#ifndef REGCMP
-			fprintf(stderr, "%s\n", pat);
-#else	/* REGCMP */
-			fprintf(stderr, "bad pattern: %s\n", pat);
-#endif	/* REGCMP */
+#ifdef HAVE_REGCMP
+			warnx("bad pattern: %s\n", pat);
+#else	/* !HAVE_REGCMP */
+			warnx("%s\n", pat);
+#endif	/* !HAVE_REGCMP */
+			RE_FREE();
 		}
 	}
 # endif	/* NO_REGEX */
@@ -393,19 +414,20 @@ register char	**argv;
  */
 int
 form_file_list(files, file_cnt)
-register char	**files;
-register int	file_cnt;
+	char	**files;
+	int	file_cnt;
 {
-	register int	i, percent;
-	register char	*sp;
+	int	i, percent;
+	const char	*sp;
 
-	if (file_cnt == 0)
+	if (file_cnt == 0) {
 		if (Find_files)
 			return add_file(NO_PROB, FORTDIR, NULL, &File_list,
 					&File_tail, NULL);
 		else
 			return add_file(NO_PROB, "fortunes", FORTDIR,
 					&File_list, &File_tail, NULL);
+	}
 	for (i = 0; i < file_cnt; i++) {
 		percent = NO_PROB;
 		if (!isdigit(files[i][0]))
@@ -415,11 +437,11 @@ register int	file_cnt;
 			for (sp = files[i]; isdigit(*sp); sp++)
 				percent = percent * 10 + *sp - '0';
 			if (percent > 100) {
-				fprintf(stderr, "percentages must be <= 100\n");
+				warnx("Percentages must be <= 100");
 				return FALSE;
 			}
 			if (*sp == '.') {
-				fprintf(stderr, "percentages must be integers\n");
+				warnx("Percentages must be integers");
 				return FALSE;
 			}
 			/*
@@ -433,7 +455,7 @@ register int	file_cnt;
 			}
 			else if (*++sp == '\0') {
 				if (++i >= file_cnt) {
-					fprintf(stderr, "percentages must precede files\n");
+					warnx("Percentages must precede files");
 					return FALSE;
 				}
 				sp = files[i];
@@ -453,30 +475,33 @@ register int	file_cnt;
  */
 int
 add_file(percent, file, dir, head, tail, parent)
-int		percent;
-register char	*file;
-char		*dir;
-FILEDESC	**head, **tail;
-FILEDESC	*parent;
+	int		 percent;
+	const char	*file;
+	const char	*dir;
+	FILEDESC	**head, **tail;
+	FILEDESC	*parent;
 {
-	register FILEDESC	*fp;
-	register int		fd;
-	register char		*path, *offensive;
-	register bool		was_malloc;
-	register bool		isdir;
+	FILEDESC	*fp;
+	int		fd;
+	const char	*path;
+	char		*tpath, *offensive;
+	bool		was_malloc;
+	bool		isdir;
 
 	if (dir == NULL) {
 		path = file;
+		tpath = NULL;
 		was_malloc = FALSE;
 	}
 	else {
-		path = do_malloc((unsigned int) (strlen(dir) + strlen(file) + 2));
-		(void) strcat(strcat(strcpy(path, dir), "/"), file);
+		tpath = do_malloc((unsigned int) (strlen(dir) + strlen(file) + 2));
+		(void) strcat(strcat(strcpy(tpath, dir), "/"), file);
+		path = tpath;
 		was_malloc = TRUE;
 	}
 	if ((isdir = is_dir(path)) && parent != NULL) {
 		if (was_malloc)
-			free(path);
+			free(tpath);
 		return FALSE;	/* don't recurse */
 	}
 	offensive = NULL;
@@ -486,7 +511,7 @@ FILEDESC	*parent;
 		was_malloc = TRUE;
 		if (Offend) {
 			if (was_malloc)
-				free(path);
+				free(tpath);
 			path = offensive;
 			file = off_name(file);
 		}
@@ -494,7 +519,7 @@ FILEDESC	*parent;
 
 	DPRINTF(1, (stderr, "adding file \"%s\"\n", path));
 over:
-	if ((fd = open(path, 0)) < 0) {
+	if ((fd = open(path, O_RDONLY)) < 0) {
 		/*
 		 * This is a sneak.  If the user said -a, and if the
 		 * file we're given isn't a file, we check to see if
@@ -506,7 +531,7 @@ over:
 		if (All_forts && offensive != NULL) {
 			path = offensive;
 			if (was_malloc)
-				free(path);
+				free(tpath);
 			offensive = NULL;
 			was_malloc = TRUE;
 			DPRINTF(1, (stderr, "\ttrying \"%s\"\n", path));
@@ -517,9 +542,9 @@ over:
 			return add_file(percent, file, FORTDIR, head, tail,
 					parent);
 		if (parent == NULL)
-			perror(path);
+			warn("Cannot open `%s'", path);
 		if (was_malloc)
-			free(path);
+			free(tpath);
 		return FALSE;
 	}
 
@@ -537,12 +562,10 @@ over:
 	     !is_fortfile(path, &fp->datfile, &fp->posfile, (parent != NULL))))
 	{
 		if (parent == NULL)
-			fprintf(stderr,
-				"fortune:%s not a fortune file or directory\n",
-				path);
+			warnx("`%s' not a fortune file or directory", path);
 		free((char *) fp);
 		if (was_malloc)
-			free(path);
+			free(tpath);
 		do_free(fp->datfile);
 		do_free(fp->posfile);
 		do_free(offensive);
@@ -582,7 +605,7 @@ over:
 FILEDESC *
 new_fp()
 {
-	register FILEDESC	*fp;
+	FILEDESC	*fp;
 
 	fp = (FILEDESC *) do_malloc(sizeof *fp);
 	fp->datfd = -1;
@@ -606,7 +629,7 @@ new_fp()
  */
 char *
 off_name(file)
-char	*file;
+	const char	*file;
 {
 	char	*new;
 
@@ -620,7 +643,7 @@ char	*file;
  */
 int
 is_off_name(file)
-char	*file;
+	const char	*file;
 {
 	int	len;
 
@@ -635,19 +658,19 @@ char	*file;
  */
 void
 all_forts(fp, offensive)
-register FILEDESC	*fp;
-char			*offensive;
+	FILEDESC	*fp;
+	const char	*offensive;
 {
-	register char		*sp;
-	register FILEDESC	*scene, *obscene;
-	register int		fd;
-	auto char		*datfile, *posfile;
+	char		*sp;
+	FILEDESC	*scene, *obscene;
+	int		 fd;
+	char		*datfile, *posfile;
 
 	if (fp->child != NULL)	/* this is a directory, not a file */
 		return;
 	if (!is_fortfile(offensive, &datfile, &posfile, FALSE))
 		return;
-	if ((fd = open(offensive, 0)) < 0)
+	if ((fd = open(offensive, O_RDONLY)) < 0)
 		return;
 	DPRINTF(1, (stderr, "adding \"%s\" because of -a\n", offensive));
 	scene = new_fp();
@@ -685,34 +708,33 @@ char			*offensive;
  */
 int
 add_dir(fp)
-register FILEDESC	*fp;
+	FILEDESC	*fp;
 {
-	register DIR		*dir;
-	register struct dirent  *dirent;
-	auto FILEDESC		*tailp;
-	auto char		*name;
+	DIR		*dir;
+	struct dirent	*dirent;
+	FILEDESC	*tailp;
+	char		*name;
 
 	(void) close(fp->fd);
 	fp->fd = -1;
 	if ((dir = opendir(fp->path)) == NULL) {
-		perror(fp->path);
+		warn("Cannot open `%s'", fp->path);
 		return FALSE;
 	}
 	tailp = NULL;
 	DPRINTF(1, (stderr, "adding dir \"%s\"\n", fp->path));
 	fp->num_children = 0;
 	while ((dirent = readdir(dir)) != NULL) {
-		if (dirent->d_namlen == 0)
+		if (NAMLEN(dirent) == 0)
 			continue;
-		name = copy(dirent->d_name, dirent->d_namlen);
+		name = copy(dirent->d_name, NAMLEN(dirent));
 		if (add_file(NO_PROB, name, fp->path, &fp->child, &tailp, fp))
 			fp->num_children++;
 		else
 			free(name);
 	}
 	if (fp->num_children == 0) {
-		(void) fprintf(stderr,
-		    "fortune: %s: No fortune files in directory.\n", fp->path);
+		warnx("`%s': No fortune files in directory.\n", fp->path);
 		return FALSE;
 	}
 	return TRUE;
@@ -724,13 +746,13 @@ register FILEDESC	*fp;
  */
 int
 is_dir(file)
-char	*file;
+	const char	*file;
 {
-	auto struct stat	sbuf;
+	struct stat	sbuf;
 
 	if (stat(file, &sbuf) < 0)
 		return FALSE;
-	return (sbuf.st_mode & S_IFDIR);
+	return (S_ISDIR(sbuf.st_mode));
 }
 
 /*
@@ -743,13 +765,18 @@ char	*file;
 /* ARGSUSED */
 int
 is_fortfile(file, datp, posp, check_for_offend)
-char	*file, **datp, **posp;
-int	check_for_offend;
+	const char	*file;
+	char		**datp, **posp
+# ifndef OK_TO_WRITE_DISK
+	__attribute__((__unused__))
+# endif
+	;
+	int	check_for_offend;
 {
-	register int	i;
-	register char	*sp;
-	register char	*datfile;
-	static char	*suflist[] = {	/* list of "illegal" suffixes" */
+	int	i;
+	const char	*sp;
+	char	*datfile;
+	static const char	*const suflist[] = {	/* list of "illegal" suffixes" */
 				"dat", "pos", "c", "h", "p", "i", "f",
 				"pas", "ftn", "ins.c", "ins,pas",
 				"ins.ftn", "sml",
@@ -812,8 +839,8 @@ int	check_for_offend;
  */
 char *
 copy(str, len)
-char		*str;
-unsigned int	len;
+	const char	*str;
+	unsigned int	len;
 {
 	char	*new, *sp;
 
@@ -831,14 +858,12 @@ unsigned int	len;
  */
 void *
 do_malloc(size)
-unsigned int	size;
+	unsigned int	size;
 {
 	void	*new;
 
-	if ((new = malloc(size)) == NULL) {
-		(void) fprintf(stderr, "fortune: out of memory.\n");
-		exit(1);
-	}
+	if ((new = malloc(size)) == NULL)
+		err(1, NULL);
 	return new;
 }
 
@@ -848,7 +873,7 @@ unsigned int	size;
  */
 void
 do_free(ptr)
-void	*ptr;
+	void	*ptr;
 {
 	if (ptr != NULL)
 		free(ptr);
@@ -861,9 +886,10 @@ void	*ptr;
 void
 init_prob()
 {
-	register FILEDESC	*fp, *last;
-	register int		percent, num_noprob, frac;
+	FILEDESC	*fp, *last;
+	int		percent, num_noprob, frac;
 
+	last = NULL;
 	/*
 	 * Distribute the residual probability (if any) across all
 	 * files with unspecified probability (i.e., probability of 0)
@@ -877,29 +903,19 @@ init_prob()
 			num_noprob++;
 			if (Equal_probs)
 				last = fp;
-		}
-		else
+		} else
 			percent += fp->percent;
 	DPRINTF(1, (stderr, "summing probabilities:%d%% with %d NO_PROB's",
 		    percent, num_noprob));
-	if (percent > 100) {
-		(void) fprintf(stderr,
-		    "fortune: probabilities sum to %d%%!\n", percent);
-		exit(1);
-	}
-	else if (percent < 100 && num_noprob == 0) {
-		(void) fprintf(stderr,
-		    "fortune: no place to put residual probability (%d%%)\n",
+	if (percent > 100)
+		errx(1, "Probabilities sum to %d%%!", percent);
+	else if (percent < 100 && num_noprob == 0)
+		errx(1, "No place to put residual probability (%d%%)",
 		    percent);
-		exit(1);
-	}
-	else if (percent == 100 && num_noprob != 0) {
-		(void) fprintf(stderr,
-		    "fortune: no probability left to put in residual files\n");
-		exit(1);
-	}
+	else if (percent == 100 && num_noprob != 0)
+		errx(1, "No probability left to put in residual files");
 	percent = 100 - percent;
-	if (Equal_probs)
+	if (Equal_probs) {
 		if (num_noprob != 0) {
 			if (num_noprob > 1) {
 				frac = percent / num_noprob;
@@ -913,7 +929,7 @@ init_prob()
 			last->percent = percent;
 			DPRINTF(1, (stderr, ", residual = %d%%", percent));
 		}
-	else {
+	} else {
 		DPRINTF(1, (stderr,
 			    ", %d%% distributed over remaining fortunes\n",
 			    percent));
@@ -933,8 +949,8 @@ init_prob()
 void
 get_fort()
 {
-	register FILEDESC	*fp;
-	register int		choice;
+	FILEDESC	*fp;
+	int		choice;
 
 	if (File_list->next == NULL || File_list->percent == NO_PROB)
 		fp = File_list;
@@ -962,7 +978,7 @@ get_fort()
 			choice = random() % Noprob_tbl.str_numstr;
 			DPRINTF(1, (stderr, "choice = %d (of %d) \n", choice,
 				    Noprob_tbl.str_numstr));
-			while (choice >= fp->tbl.str_numstr) {
+			while ((u_int32_t)choice >= fp->tbl.str_numstr) {
 				choice -= fp->tbl.str_numstr;
 				fp = fp->next;
 				DPRINTF(1, (stderr,
@@ -983,10 +999,10 @@ get_fort()
 	get_pos(fp);
 	open_dat(fp);
 	(void) lseek(fp->datfd,
-		     (off_t) (sizeof fp->tbl + fp->pos * sizeof Seekpts[0]), 0);
+		     (off_t) (sizeof fp->tbl + fp->pos * sizeof Seekpts[0]), SEEK_SET);
 	read(fp->datfd, Seekpts, sizeof Seekpts);
-	Seekpts[0] = ntohl(Seekpts[0]);
-	Seekpts[1] = ntohl(Seekpts[1]);
+	BE64TOH(Seekpts[0]);
+	BE64TOH(Seekpts[1]);
 }
 
 /*
@@ -995,10 +1011,10 @@ get_fort()
  */
 FILEDESC *
 pick_child(parent)
-FILEDESC	*parent;
+	FILEDESC	*parent;
 {
-	register FILEDESC	*fp;
-	register int		choice;
+	FILEDESC	*fp;
+	int		 choice;
 
 	if (Equal_probs) {
 		choice = random() % parent->num_children;
@@ -1014,7 +1030,7 @@ FILEDESC	*parent;
 		choice = random() % parent->tbl.str_numstr;
 		DPRINTF(1, (stderr, "    choice = %d (of %d)\n",
 			    choice, parent->tbl.str_numstr));
-		for (fp = parent->child; choice >= fp->tbl.str_numstr;
+		for (fp = parent->child; (u_int32_t)choice >= fp->tbl.str_numstr;
 		     fp = fp->next) {
 			choice -= fp->tbl.str_numstr;
 			DPRINTF(1, (stderr, "\tskip %s, %d (choice = %d)\n",
@@ -1032,7 +1048,7 @@ FILEDESC	*parent;
  */
 void
 sum_noprobs(fp)
-register FILEDESC	*fp;
+	FILEDESC	*fp;
 {
 	static bool	did_noprobs = FALSE;
 
@@ -1049,7 +1065,7 @@ register FILEDESC	*fp;
 
 int
 max(i, j)
-register int	i, j;
+	int	i, j;
 {
 	return (i >= j ? i : j);
 }
@@ -1060,12 +1076,10 @@ register int	i, j;
  */
 void
 open_fp(fp)
-FILEDESC	*fp;
+	FILEDESC	*fp;
 {
-	if (fp->inf == NULL && (fp->inf = fdopen(fp->fd, "r")) == NULL) {
-		perror(fp->path);
-		exit(1);
-	}
+	if (fp->inf == NULL && (fp->inf = fdopen(fp->fd, "r")) == NULL)
+		err(1, "Cannot open `%s'", fp->path);
 }
 
 /*
@@ -1074,12 +1088,10 @@ FILEDESC	*fp;
  */
 void
 open_dat(fp)
-FILEDESC	*fp;
+	FILEDESC	*fp;
 {
-	if (fp->datfd < 0 && (fp->datfd = open(fp->datfile, 0)) < 0) {
-		perror(fp->datfile);
-		exit(1);
-	}
+	if (fp->datfd < 0 && (fp->datfd = open(fp->datfile, O_RDONLY)) < 0)
+		err(1, "Cannot open `%s'", fp->datfile);
 }
 
 /*
@@ -1089,7 +1101,7 @@ FILEDESC	*fp;
  */
 void
 get_pos(fp)
-FILEDESC	*fp;
+	FILEDESC	*fp;
 {
 #ifdef	OK_TO_WRITE_DISK
 	int	fd;
@@ -1098,7 +1110,7 @@ FILEDESC	*fp;
 	assert(fp->read_tbl);
 	if (fp->pos == POS_UNKNOWN) {
 #ifdef	OK_TO_WRITE_DISK
-		if ((fd = open(fp->posfile, 0)) < 0 ||
+		if ((fd = open(fp->posfile, O_RDONLY)) < 0 ||
 		    read(fd, &fp->pos, sizeof fp->pos) != sizeof fp->pos)
 			fp->pos = random() % fp->tbl.str_numstr;
 		else if (fp->pos >= fp->tbl.str_numstr)
@@ -1109,7 +1121,7 @@ FILEDESC	*fp;
 		fp->pos = random() % fp->tbl.str_numstr;
 #endif /* OK_TO_WRITE_DISK */
 	}
-	if (++(fp->pos) >= fp->tbl.str_numstr)
+	if ((u_int64_t)++(fp->pos) >= fp->tbl.str_numstr)
 		fp->pos -= fp->tbl.str_numstr;
 	DPRINTF(1, (stderr, "pos for %s is %qd\n", fp->name, fp->pos));
 }
@@ -1120,28 +1132,24 @@ FILEDESC	*fp;
  */
 void
 get_tbl(fp)
-FILEDESC	*fp;
+	FILEDESC	*fp;
 {
-	auto int		fd;
-	register FILEDESC	*child;
+	int	 	fd;
+	FILEDESC	*child;
 
 	if (fp->read_tbl)
 		return;
 	if (fp->child == NULL) {
-		if ((fd = open(fp->datfile, 0)) < 0) {
-			perror(fp->datfile);
-			exit(1);
-		}
+		if ((fd = open(fp->datfile, O_RDONLY)) < 0)
+			err(1, "Cannot open `%s'", fp->datfile);
 		if (read(fd, (char *) &fp->tbl, sizeof fp->tbl) != sizeof fp->tbl) {
-			(void)fprintf(stderr,
-			    "fortune: %s corrupted\n", fp->path);
-			exit(1);
+			errx(1, "Database `%s' corrupted", fp->path);
 		}
-		/* fp->tbl.str_version = ntohl(fp->tbl.str_version); */
-		fp->tbl.str_numstr = ntohl(fp->tbl.str_numstr);
-		fp->tbl.str_longlen = ntohl(fp->tbl.str_longlen);
-		fp->tbl.str_shortlen = ntohl(fp->tbl.str_shortlen);
-		fp->tbl.str_flags = ntohl(fp->tbl.str_flags);
+		/* BE32TOH(fp->tbl.str_version); */
+		BE32TOH(fp->tbl.str_numstr);
+		BE32TOH(fp->tbl.str_longlen);
+		BE32TOH(fp->tbl.str_shortlen);
+		BE32TOH(fp->tbl.str_flags);
 		(void) close(fd);
 	}
 	else {
@@ -1160,7 +1168,7 @@ FILEDESC	*fp;
  */
 void
 zero_tbl(tp)
-register STRFILE	*tp;
+	STRFILE	*tp;
 {
 	tp->str_numstr = 0;
 	tp->str_longlen = 0;
@@ -1173,7 +1181,7 @@ register STRFILE	*tp;
  */
 void
 sum_tbl(t1, t2)
-register STRFILE	*t1, *t2;
+	STRFILE	*t1, *t2;
 {
 	t1->str_numstr += t2->str_numstr;
 	if (t1->str_longlen < t2->str_longlen)
@@ -1200,8 +1208,8 @@ print_file_list()
  */
 void
 print_list(list, lev)
-register FILEDESC	*list;
-int			lev;
+	FILEDESC	*list;
+	int		 lev;
 {
 	while (list != NULL) {
 		fprintf(stderr, "%*s", lev * 4, "");
@@ -1226,11 +1234,11 @@ int			lev;
  */
 char *
 conv_pat(orig)
-register char	*orig;
+	char	*orig;
 {
-	register char		*sp;
-	register unsigned int	cnt;
-	register char		*new;
+	char		*sp;
+	unsigned int	 cnt;
+	char		*new;
 
 	cnt = 1;	/* allow for '\0' */
 	for (sp = orig; *sp != '\0'; sp++)
@@ -1238,10 +1246,8 @@ register char	*orig;
 			cnt += 4;
 		else
 			cnt++;
-	if ((new = malloc(cnt)) == NULL) {
-		fprintf(stderr, "pattern too long for ignoring case\n");
-		exit(1);
-	}
+	if ((new = malloc(cnt)) == NULL)
+		err(1, NULL);
 
 	for (sp = new; *orig != '\0'; orig++) {
 		if (islower(*orig)) {
@@ -1287,10 +1293,10 @@ find_matches()
  */
 int
 maxlen_in_list(list)
-FILEDESC	*list;
+	FILEDESC	*list;
 {
-	register FILEDESC	*fp;
-	register int		len, maxlen;
+	FILEDESC	*fp;
+	int		 len, maxlen;
 
 	maxlen = 0;
 	for (fp = list; fp != NULL; fp = fp->next) {
@@ -1300,7 +1306,7 @@ FILEDESC	*list;
 		}
 		else {
 			get_tbl(fp);
-			if (fp->tbl.str_longlen > maxlen)
+			if (fp->tbl.str_longlen > (u_int32_t)maxlen)
 				maxlen = fp->tbl.str_longlen;
 		}
 	}
@@ -1313,11 +1319,11 @@ FILEDESC	*list;
  */
 void
 matches_in_list(list)
-FILEDESC	*list;
+	FILEDESC	*list;
 {
-	register char		*sp;
-	register FILEDESC	*fp;
-	int			in_file;
+	char		*sp;
+	FILEDESC	*fp;
+	int		 in_file;
 
 	for (fp = list; fp != NULL; fp = fp->next) {
 		if (fp->child != NULL) {
@@ -1347,13 +1353,15 @@ FILEDESC	*list;
 				sp = Fortbuf;
 			}
 	}
+	RE_FREE();
 }
 # endif	/* NO_REGEX */
 
 void
 usage()
 {
-	(void) fprintf(stderr, "fortune [-a");
+	extern char *__progname;
+	(void) fprintf(stderr, "%s [-a", __progname);
 #ifdef	DEBUG
 	(void) fprintf(stderr, "D");
 #endif	/* DEBUG */
